@@ -14,7 +14,6 @@
 # limitations under the License.
 
 import re
-import six
 
 from oslo_log import log as logging
 from oslo_utils import encodeutils
@@ -284,7 +283,7 @@ class SaharaNodeGroupTemplate(resource.Resource):
         return props
 
     def handle_create(self):
-        props = dict((k, v) for k, v in six.iteritems(self.properties))
+        props = dict((k, v) for k, v in self.properties.items())
         args = self._prepare_properties(props)
         node_group_template = self.client().node_group_templates.create(**args)
         LOG.info("Node Group Template '%s' has been created",
@@ -335,7 +334,7 @@ class SaharaNodeGroupTemplate(resource.Resource):
             self.properties[self.PLUGIN_NAME],
             self.properties[self.HADOOP_VERSION])
         allowed_processes = [item for sublist in
-                             list(six.itervalues(plugin.node_processes))
+                             list(plugin.node_processes.values())
                              for item in sublist]
         unsupported_processes = []
         for process in self.properties[self.NODE_PROCESSES]:
@@ -564,7 +563,7 @@ class SaharaClusterTemplate(resource.Resource):
         return props
 
     def handle_create(self):
-        props = dict((k, v) for k, v in six.iteritems(self.properties))
+        props = dict((k, v) for k, v in self.properties.items())
         args = self._prepare_properties(props)
         cluster_template = self.client().cluster_templates.create(**args)
         LOG.info("Cluster Template '%s' has been created",
